package pgdp.trees;

import java.util.Comparator;
import java.util.function.Predicate;

public class Leaf<T> implements TreeElement<T> {

	@Override
	public TreeElement<T> insert(T value, Comparator<T> comp) {
		// Blatt durch innerNode ersetzen und an Eltern zurückgeben
		return new InnerNode<T>(value);
	}

	@Override
	public void toString(StringBuilder sb) {
		// Blatt is leer, also wird der String nicht verändert
		return;

	}

	@Override
	public int size() {
		// Blatt hat keine Kinder mehr, also Länge 0
		return 0;

	}

	@Override
	public T getMin() {
		// Es gibt im Teilbaum von Blättern (leer) kein minimales Element
		return null;
	}

	@Override
	public TreeElement<T> remove(T value, Comparator<T> comp) {
		// es wird nichts verändert, entsprechend wieder das Blatt zurückgeben
		return this;
	}

	@Override
	public boolean contains(T value, Comparator<T> comp) {
		// Blatt kann nichts enhalten
		return false;
	}

	@Override
	public int countMatches(Predicate<T> filter) {
		// Blatt enthält nichts, kann also auch keine Bedingungen erfüllen
		return 0;
	}

	@Override
	public int getAll(Predicate<T> filter, T[] array, int index) {
		// Blatt enhält nichts, keine Veränderung, unveränderten Index zurückgeben
		return index;
	}
}
