package pgdp.trees;

import java.util.Comparator;
import java.util.function.Predicate;

public class PenguinTrees {
	public static Tree<Penguin> getAgeTree() {

		// Comparator für Alter definieren
		Comparator<Penguin> comp = new Comparator<Penguin>() {

			@Override
			public int compare(Penguin o1, Penguin o2) {
				int ageDiff = o1.getAge() - o2.getAge();
				return ageDiff;
			}

		};

		// Baum erstellen
		Tree<Penguin> tree = new Tree<Penguin>(comp);
		
		/*
		 * Alternativ mit Lamda Expression
		 * Tree<Penguin> tree = new Tree<Penguin>((o1, o2) -> (o1.getAge() - o2.getAge()));
		 */

		return tree;
	}

	public static Tree<Penguin> getHeightTree() {

		// Comparator für Größe definieren
		Comparator<Penguin> comp = new Comparator<Penguin>() {

			@Override
			public int compare(Penguin o1, Penguin o2) {
				// Vorzeichen anschauen (Math.signum gibt 0, +1 oder -1 zurück), ein double 0.1
				// wäre int 0 > nicht das gewünschte Vergleichsverhalten
				int heightDiff = (int) Math.signum(o2.getHeight() - o1.getHeight());
				return heightDiff;
			}

		};

		// Baum erstellen
		Tree<Penguin> tree = new Tree<Penguin>(comp);

		return tree;
	}

	public static int getMinorsAmount(Tree<Penguin> tree) {
		// Lamda Expression
		return tree.countMatches(p -> p.getAge() < 5);
	}

	public static Object[] getEvenNames(Tree<Penguin> tree) {

		// Prädikat manuell erstellen
		Predicate<Penguin> filter = new Predicate<Penguin>() {

			@Override
			public boolean test(Penguin t) {
				return ((t.getName().length() % 2) == 0);
			}

		};

		// Prädikat anwenden
		return tree.getAll(filter);
		
		/*
		 * alternativ mit Lamda Expression:
		 * return tree.getAll(p -> (p.getName().length() % 2) == 0);
		 */
	}
}
