package pgdp.trees;

import java.util.Comparator;
import java.util.function.Predicate;

public class Tree<T> {

	private Comparator<T> comp;
	private TreeElement<T> root;

	public Tree(Comparator<T> comp) {
		this.comp = comp;
		this.root = new Leaf<T>();
	}

	public void insert(T value) {
		if (value == null)
			return;

		// Falls der Baum leer ist (Leaf), wird ein innerer Knoten initialisiert
		if (this.root instanceof Leaf)
			this.root = new InnerNode<T>(value);

		// falls der Baum nicht leer ist, wird das Einfügen an den Wurzelknoten
		// weitergegeben
		else
			this.root.insert(value, this.comp);
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		// Stringbuilder in den Baum geben (durchreichen), um den String zu bauen
		this.root.toString(sb);
		return sb.toString();

	}

	public int size() {
		return this.root.size();
	}

	public void remove(T value) {
		// ab Wurzel nach dem Element suchen (und es dann löschen)
		this.root.remove(value, this.comp);
	}

	public boolean contains(T value) {
		// ab Wurzel nach dem Element suchen
		return this.root.contains(value, this.comp);
	}

	public int countMatches(Predicate<T> filter) {
		// ab Wurzel den Baum mit dem Filter durchsuchen
		return this.root.countMatches(filter);
	}

	public T[] getAll(Predicate<T> filter) {
		// Generics und Arrays vertragen sich nicht besonders gut
		@SuppressWarnings("unchecked")
		// cast von Object zu generischem Datentyp
		T[] array = (T[]) new Object[countMatches(filter)];
		int index = 0;
		root.getAll(filter, array, index);
		return array;
	}
}
